package com.ruoyi.web.controller.care;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.ruoyi.care.domain.CareApplications;
import com.ruoyi.care.domain.CarePersonDie;
import com.ruoyi.care.service.ICareApplicationsService;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.bean.InstanceUtil;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 护理申请Controller
 * 
 * @author qy
 * @date 2024-11-25
 */
@RestController
@RequestMapping("/care/applications")
public class CareApplicationsController extends BaseController
{
    @Autowired
    private ICareApplicationsService careApplicationsService;

    /**
     * 查询护理申请列表
     */
    @PreAuthorize("@ss.hasPermi('care:applications:list')")
    @DataScope(deptAlias = "u", userAlias = "u")
    @GetMapping("/list")
    public TableDataInfo list(CareApplications careApplications)
    {
        startPage();
        List<CareApplications> list = careApplicationsService.selectCareApplicationsList(careApplications);
        return getDataTable(list);
    }

    /**
     * 导出护理申请列表
     */
    @PreAuthorize("@ss.hasPermi('care:applications:export')")
    @Log(title = "护理申请", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CareApplications careApplications)
    {
        List<CareApplications> list = careApplicationsService.selectCareApplicationsList(careApplications);
        ExcelUtil<CareApplications> util = new ExcelUtil<CareApplications>(CareApplications.class);
        util.exportExcel(response, list, "护理申请数据");
    }

    /**
     * 获取护理申请详细信息
     */
    @PreAuthorize("@ss.hasPermi('care:applications:query')")
    @GetMapping(value = "/{applyId}")
    public AjaxResult getInfo(@PathVariable("applyId") Long applyId)
    {
        return success(careApplicationsService.selectCareApplicationsByApplyId(applyId));
    }

    /**
     * 新增护理申请
     */
    @PreAuthorize("@ss.hasPermi('care:applications:add')")
    @Log(title = "护理申请", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody CareApplications careApplications)
    {
        //判断当前申请日期是否有护理记录或正在进行的护理记录
        CareApplications careApplications1=new CareApplications();
        careApplications1.setPersonId(careApplications.getPersonId());
        careApplications1.setApplyDate(careApplications.getApplyDate());
        careApplications1.setNotDealStatus("complete");
        List<CareApplications> lst=careApplicationsService.selectCareApplicationsList(careApplications1);
        if(lst.size()>0){
            return AjaxResult.error("当前特困人员在该申请日期已有未完成的护理申请");
        }
        careApplications.setCreateBy(getUserId().toString());
        //默认的状态
        if(StringUtils.isBlank(careApplications.getDealStatus())){
            careApplications.setDealStatus("no_start");
        }
        if(StringUtils.isBlank(careApplications.getStatus())){
            careApplications.setStatus("no_check");
        }
        if(StringUtils.isBlank(careApplications.getPayStatus())){
            careApplications.setPayStatus("dis_pay");
        }
        return toAjax(careApplicationsService.insertCareApplications(careApplications));
    }

    /**
     * 修改护理申请
     */
    @PreAuthorize("@ss.hasPermi('care:applications:edit')")
    @Log(title = "护理申请", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody CareApplications careApplications)
    {
        //查询数据库的护理申请
        if(StringUtils.isBlank(careApplications.getApplyId().toString())){
            return AjaxResult.error("修改的数据未提交标识");
        }
        CareApplications oldApp=careApplicationsService.selectCareApplicationsByApplyId(careApplications.getApplyId());
        //判断数据这个护理申请是否在流程中或者已完成
        if(!(oldApp.getStatus().equals("no_check") && oldApp.getDealStatus().equals("no_start"))){
            return AjaxResult.error("当前这条数据已经在流程中，无法进行修改");
        }
        //旧数据和新数据进行数据合并
       /* try {
            careApplications = InstanceUtil.mergeDeviceProperties(careApplications, oldApp);
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        }*/
        careApplications.setUpdateBy(getUserId().toString());
        return toAjax(careApplicationsService.updateCareApplications(careApplications));
    }

    /**
     * 删除护理申请
     */
    @PreAuthorize("@ss.hasPermi('care:applications:remove')")
    @Log(title = "护理申请", businessType = BusinessType.DELETE)
	@DeleteMapping("/{applyIds}")
    public AjaxResult remove(@PathVariable Long[] applyIds)
    {
        return toAjax(careApplicationsService.deleteCareApplicationsByApplyIds(applyIds));
    }

    /**
     * 送审护理申请
     */
    @PreAuthorize("@ss.hasPermi('care:applications:edit')")
    @Log(title = "护理送审", businessType = BusinessType.UPDATE)
    @PostMapping("/start")
    public AjaxResult start(@RequestBody CareApplications careApplications)
    {
        Long userId=getUserId();
        return toAjax(careApplicationsService.start(careApplications,userId));
    }

    /**
     * 审核护理申请
     */
    @PreAuthorize("@ss.hasPermi('care:applications:check')")
    @Log(title = "护理审核", businessType = BusinessType.UPDATE)
    @PostMapping("/check")
    public AjaxResult check(@RequestBody CareApplications careApplications)
    {
        Long userId=getUserId();
        return toAjax(careApplicationsService.check(careApplications,userId));
    }


    /**
     * 指派护理申请
     */
    @Log(title = "护理指派公司", businessType = BusinessType.UPDATE)
    @PostMapping("/assign")
    public AjaxResult assign(@RequestBody CareApplications careApplications)
    {
        Long userId=getUserId();
        return toAjax(careApplicationsService.assign(careApplications,userId));
    }

    /**
     * 护理完成
     */
    @Log(title = "护理完成", businessType = BusinessType.UPDATE)
    @PostMapping("/complete")
    public AjaxResult complete(@RequestBody CareApplications careApplications)
    {
        Long userId=getUserId();
        return toAjax(careApplicationsService.complete(careApplications,userId));
    }

    /**
     * 指派人员列表
     */
    @PreAuthorize("@ss.hasPermi('care:applications:assignList')")
    @GetMapping("/assignList")
    public TableDataInfo assignList(CareApplications careApplications)
    {
        startPage();
        Long userId=getUserId();
        careApplications.setCompanyUserId(userId);
        List<CareApplications> list = careApplicationsService.selectCareApplicationsList(careApplications);
        return getDataTable(list);
    }

    /**
     * 任务打卡列表
     */
    @PreAuthorize("@ss.hasPermi('care:applications:clockList')")
    @GetMapping("/clockList")
    public TableDataInfo clockList(CareApplications careApplications)
    {
        startPage();
        Long userId=getUserId();
        careApplications.setNurseUserId(userId);
        List<CareApplications> list = careApplicationsService.selectCareApplicationsList(careApplications);
        return getDataTable(list);
    }

    /**
     * 评分列表
     */
    @PreAuthorize("@ss.hasPermi('care:applications:scoreList')")
    @GetMapping("/scoreList")
    @DataScope(deptAlias = "u", userAlias = "u")
    public TableDataInfo scoreList(CareApplications careApplications)
    {
        startPage();
        careApplications.setDealStatus("complete");
        careApplications.setStatus("assigned");
        List<CareApplications> list = careApplicationsService.selectCareApplicationsList(careApplications);
        return getDataTable(list);
    }
}
