package com.ruoyi.web.controller.system;

import java.util.List;
import java.util.Map;

import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.system.domain.SysNoticeUser;
import com.ruoyi.system.service.ISysNoticeUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.SysNotice;
import com.ruoyi.system.service.ISysNoticeService;

/**
 * 公告 信息操作处理
 * 
 * @author ruoyi
 */
@RestController
@RequestMapping("/system/notice")
public class SysNoticeController extends BaseController
{
    @Autowired
    private ISysNoticeService noticeService;
    @Autowired
    private ISysNoticeUserService noticeUserService;

    /**
     * 获取通知公告列表
     */
    @PreAuthorize("@ss.hasPermi('system:notice:list')")
    @GetMapping("/list")
    public TableDataInfo list(SysNotice notice)
    {
        startPage();
        List<SysNotice> list = noticeService.selectNoticeList(notice);
        return getDataTable(list);
    }

    /**
     * 根据通知公告编号获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:notice:query')")
    @GetMapping(value = "/{noticeId}")
    public AjaxResult getInfo(@PathVariable Long noticeId)
    {
        return success(noticeService.selectNoticeById(noticeId));
    }

    /**
     * 新增通知公告
     */
    @PreAuthorize("@ss.hasPermi('system:notice:add')")
    @Log(title = "通知公告", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody SysNotice notice)
    {
        notice.setCreateBy(getUsername());
        return toAjax(noticeService.insertNotice(notice));
    }

    /**
     * 修改通知公告
     */
    @PreAuthorize("@ss.hasPermi('system:notice:edit')")
    @Log(title = "通知公告", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody SysNotice notice)
    {
        notice.setUpdateBy(getUsername());
        return toAjax(noticeService.updateNotice(notice));
    }

    /**
     * 删除通知公告
     */
    @PreAuthorize("@ss.hasPermi('system:notice:remove')")
    @Log(title = "通知公告", businessType = BusinessType.DELETE)
    @DeleteMapping("/{noticeIds}")
    public AjaxResult remove(@PathVariable Long[] noticeIds)
    {
        return toAjax(noticeService.deleteNoticeByIds(noticeIds));
    }


    /**
     * 移除人员
     */
    @PreAuthorize("@ss.hasPermi('system:notice:choose')")
    @Log(title = "移除人员", businessType = BusinessType.UPDATE)
    @GetMapping("/removeUser/{noticeUserId}")
    public AjaxResult removeUser(@PathVariable Long noticeUserId)
    {
        return toAjax(noticeUserService.deleteNoticeUserById(noticeUserId));
    }

    /*
        标记为已读
     */
    @Log(title = "标记为已读", businessType = BusinessType.UPDATE)
    @PostMapping(value ="/setIsRead")
    public AjaxResult setIsRead(@Validated @RequestBody SysNoticeUser notice)
    {
        notice.setStatus("1");//已读
        notice.setUpdateTime(DateUtils.getNowDate());
        notice.setUpdateBy(getUsername());
        return toAjax(noticeUserService.updateNoticeUser(notice));
    }

    /**
     * 获取通知公告用户列表
     */
    @GetMapping(value ="/listUser")
    public TableDataInfo listUser(SysNoticeUser notice)
    {
        startPage();
        List<SysNoticeUser> list = noticeUserService.selectNoticeList(notice);
        return getDataTable(list);
    }

    @Log(title = "添加公告用户", businessType = BusinessType.INSERT)
    @PostMapping("/addNoticeUser/{noticeId}")
    public AjaxResult addNoticeUser(@PathVariable Long noticeId, @RequestBody List<SysUser> users) {
        if (users.isEmpty()) {
            return warn("请选择用户后再添加！");
        }
        return success(noticeService.addNoticeUser(noticeId, users));
    }

    /**
     * 删除通知公告
     */
    @GetMapping("/removeNoticeUser/{noticeIds}")
    public AjaxResult removeNoticeUser(@PathVariable Long[] noticeIds)
    {
        return toAjax(noticeUserService.deleteNoticeUserByIds(noticeIds));
    }

    /**
     * 获取通知公告用户列表
     */
    @GetMapping(value ="/myNoticeList")
    public TableDataInfo myNoticeList(SysNoticeUser notice)
    {
        startPage();
        List<Map<String,Object>> list = noticeUserService.selectMyNoticeList(notice);

        return getDataTable(list);
    }
}
