package com.ruoyi.care.service.impl;

import java.io.File;
import java.io.IOException;
import java.math.BigDecimal;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import com.ruoyi.care.domain.*;
import com.ruoyi.care.mapper.*;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.enums.CareApplyStatus;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.system.mapper.SysWorkConfigMapper;
import org.apache.commons.lang3.time.FastDateFormat;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.care.service.ICareNurseClocksService;
import org.springframework.transaction.annotation.Transactional;

/**
 * 护理任务打卡Service业务层处理
 * 
 * @author qy
 * @date 2024-11-26
 */
@Service
public class CareNurseClocksServiceImpl implements ICareNurseClocksService 
{
    @Autowired
    private CareNurseClocksMapper careNurseClocksMapper;
    @Autowired
    private CareApplicationsMapper careApplicationsMapper;
    @Autowired
    private CareNurseRecordsMapper careNurseRecordsMapper;
    @Autowired
    private CareRecordsMapper careRecordsMapper;
    @Autowired
    private SysWorkConfigMapper sysWorkConfigMapper;
    @Autowired
    private CarePersonsMapper carePersonsMapper;

    /**
     * 查询护理任务打卡
     * 
     * @param clockId 护理任务打卡主键
     * @return 护理任务打卡
     */
    @Override
    public CareNurseClocks selectCareNurseClocksByClockId(Long clockId)
    {
        return careNurseClocksMapper.selectCareNurseClocksByClockId(clockId);
    }

    /**
     * 查询护理任务打卡列表
     * 
     * @param careNurseClocks 护理任务打卡
     * @return 护理任务打卡
     */
    @Override
    public List<CareNurseClocks> selectCareNurseClocksList(CareNurseClocks careNurseClocks)
    {
        return careNurseClocksMapper.selectCareNurseClocksList(careNurseClocks);
    }

    /**
     * 新增护理任务打卡
     * 
     * @param careNurseClocks 护理任务打卡
     * @return 结果
     */
    @Override
    @Transactional
    public int insertCareNurseClocks(CareNurseClocks careNurseClocks,Long userId) {
        //判断当前日期是否可以打卡，非残疾人出院前两天不再护理，残疾人截至到出院当天,若无法打卡，则抛出异常
        CareApplications careApplications = careApplicationsMapper.selectCareApplicationsByApplyId(Long.parseLong(careNurseClocks.getApplyId()));
        String leaveTimeStr = careApplications.getLeaveTime();

        // 定义日期格式化器
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

        // 将字符串转换为LocalDateTime对象
        LocalDateTime leaveTime = LocalDateTime.parse(leaveTimeStr, formatter);
        LocalDateTime currentTime = LocalDateTime.now();

        boolean isDisabled = !careApplications.getDisability().equals("N");

        if (isDisabled) {
            // 残疾人：截至到出院当天
            if (currentTime.isAfter(leaveTime)) {
                throw new ServiceException("无法打卡：当前时间已超过出院时间!");
            }
        } else {
            // 非残疾人：出院前两天不再护理
            LocalDateTime cutoffTime = leaveTime.minus(2, ChronoUnit.DAYS);
            if (currentTime.isAfter(cutoffTime)) {
                throw new ServiceException("无法打卡：非残疾人出院前两天不再提供护理服务!");
            }
        }

        //若特困人员在中转或者擅自离院，不提供护理
        CarePersons carePersons=carePersonsMapper.selectCarePersonsByPersonId(careApplications.getPersonId());
        if(carePersons.getStatus().equals("02")){
            throw new ServiceException("无法打卡：当前特困人员处于中转状态!");
        }else if(carePersons.getStatus().equals("03")){
            throw new ServiceException("无法打卡：当前特困人员处于擅自离院状态!");
        }

        String psths[]=careNurseClocks.getImgPaths().split(",");
        int result=0;
        //获取当前节点记录
        CareRecords curr=new CareRecords();
        curr.setApplyId(Long.parseLong(careNurseClocks.getApplyId()));
        curr.setState("0");
        List<CareRecords> list=careRecordsMapper.selectCareRecordsList(curr);
        if(list.size()>0){
            curr=list.get(0);
            curr.setOperate(userId.toString());
            careRecordsMapper.updateCareRecords(curr);
        }

        careNurseClocks.setCreateTime(DateUtils.getNowDate());
        careNurseClocks.setImgPaths(careNurseClocks.getImgPaths().replaceAll("/temp/","/upload/"));
        // 判断当天任务是否一开始
        String  date=careNurseClocks.getNursingTime().substring(0,10);
        CareNurseRecords careNurseRecords= new CareNurseRecords();
        careNurseRecords.setNursingTime(date);
        careNurseRecords.setApplyId(Long.parseLong(careNurseClocks.getApplyId()));
        List<CareNurseRecords> lst=careNurseRecordsMapper.selectCareNurseRecordsList(careNurseRecords);
        if(lst.size()>0){
            CareNurseRecords careNurseRecord=lst.get(0);
            careRecordsMapper.updateCareRecords(curr);

            //计算计时天数
            try {
                careNurseRecord=calDayApplyRecord(careNurseRecord,careNurseClocks.getNursingTime());
            } catch (ParseException e) {
                throw new RuntimeException(e);
            }
            System.out.print(careNurseRecord);
            careNurseRecordsMapper.updateCareNurseRecords(careNurseRecord);

           //已经开始了直接新增打卡记录
           result= careNurseClocksMapper.insertCareNurseClocks(careNurseClocks);
        }else{
           //修改申请状态为正在进行，并添加护理记录及打卡记录
           careApplications.setDealStatus("in_progress");
           careApplicationsMapper.updateCareApplications(careApplications);

           careNurseRecords=new CareNurseRecords();
           careNurseRecords.setNursingTime(careNurseClocks.getNursingTime().substring(0,10));
           careNurseRecords.setApplyId(Long.parseLong(careNurseClocks.getApplyId()));
           careNurseRecords.setClockNum(1);//当天第一次打卡
           careNurseRecords.setCalDays(new BigDecimal(0));//当天第一次打卡,计算天数为0
            careNurseRecords.setCalDayReason("今天第一次打卡，计算天数暂定为0");

            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            Date time = null;
            try {
                time = sdf.parse(careNurseClocks.getNursingTime());
            } catch (ParseException e) {
                throw new RuntimeException(e);
            }
            careNurseRecords.setCreateTime(time);
            careNurseRecordsMapper.insertCareNurseRecords(careNurseRecords);

           //获取打卡次数
            CareNurseRecords recordCount=new CareNurseRecords();
            recordCount.setApplyId(Long.parseLong(careNurseClocks.getApplyId()));
           curr.setRemark("已打卡"+careNurseRecordsMapper.selectCareNurseRecordsList(recordCount).size()+"次");
           careRecordsMapper.updateCareRecords(curr);

            result=careNurseClocksMapper.insertCareNurseClocks(careNurseClocks);
        }

        //保存成功后 将文件转移到正式路径

        for(int i=0;i<psths.length;i++){
            String path= RuoYiConfig.getProfile()+psths[i].replace("profile","");
            String finalFilePath=RuoYiConfig.getUploadPath()+psths[i].replace("/profile/temp","");
            try {
                moveFileToFinalPath(path,finalFilePath);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }
        return result;

    }

    @Override
    @Transactional
    public int die(CarePersonDie carePersonDie, Long userId) {
        //特困人员死亡后：1、保存死亡时间及死亡证明 2、修改特困人员表状态 3、计算死亡当天护理天数
        CareApplications careApplications  =careApplicationsMapper.selectCareApplicationsByApplyId(carePersonDie.getApplyId());
        careApplications.setDieTime(carePersonDie.getDieTime());
        careApplications.setDieImg(carePersonDie.getImgPaths());
        careApplications.setDealStatus("complete");
        careApplicationsMapper.updateCareApplications(careApplications);

        CarePersons persons=carePersonsMapper.selectCarePersonsByPersonId(careApplications.getPersonId());
        persons.setDieTime(carePersonDie.getDieTime());
        persons.setIsDie("01");
        carePersonsMapper.updateCarePersons(persons);

        //获取最后一次打卡记录
        CareNurseClocks careNurseClock=careNurseClocksMapper.selectLastClock(careApplications.getApplyId());
        CareNurseRecords careNurseRecord=careNurseRecordsMapper.selectCareNurseRecordsByNursingRecordId(careNurseClock.getClockId());
        //计算计时天数
        try {
            careNurseRecord=calDayApplyRecord(careNurseRecord,careNurseClock.getNursingTime());
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
        return careNurseRecordsMapper.updateCareNurseRecords(careNurseRecord);
    }

    private CareNurseRecords calDayApplyRecord(CareNurseRecords careNurseRecords,String nurseTime) throws ParseException {
        BigDecimal day=new BigDecimal(0);
        String reason="";
        //获取第一次打卡时间 判断是上午开始还是下午开始的
        Date firstTime=careNurseRecords.getCreateTime();

        // 判断是上午还是下午
        boolean isFirstClockInMorning = isMorning(firstTime);

        //获取当天打卡次数
        Integer count=careNurseRecords.getClockNum()+1;

        //获取半天次数和全天次数
        Long allday=sysWorkConfigMapper.selectSysWorkConfigByType("day_clock_num").getNumber();
        Long halfday=sysWorkConfigMapper.selectSysWorkConfigByType("half_day_num").getNumber();

        //如果第一次是上午打卡，若当前是上午且次数已经达到半天次数，则返回0.5天，若次数未达到，则等待下午打卡；若当前是下午，且次数达到一天，则返回1天
        //如果第一次是下午打卡，若当前是上午且次数已经达到半天次数，则返回0.5天，若次数未达到，则返回0

        //判断当前是上午还是下午
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date currentTime = sdf.parse(nurseTime);
        if (isFirstClockInMorning) {// 第一次打卡在上午
            if (isMorning(currentTime)) {
                if (count >= halfday) {
                    day= new BigDecimal(0.5); // 上午已达半天次数
                    reason="当天从上午开始打卡，截至最后一次打卡已达半天次数，故计算天数0.5天";
                }
            } else { // 当前是下午
                if (count >= allday) {
                    day= new BigDecimal(1); // 已达全天次数
                    reason="当天从上午开始打卡，截至最后一次打卡已达全天次数，故计算天数1天";
                } else {
                    day= new BigDecimal(0.5); // 未达到全天次数，但已过半天
                    reason="当天从上午开始打卡，截至最后一次打卡未达到全天次数，但已过半天，故计算天数0.5天";
                }
            }
        } else { // 第一次打卡在下午
            if (!isMorning(currentTime)) {// 当前也是下午
                if (count >= halfday) {
                    day= new BigDecimal(0.5); // 达到半天次数
                    reason="当天从下午开始打卡，截至最后一次打卡达到半天次数，故计算天数0.5天";
                }else{
                    reason="当天从下午开始打卡，截至最后一次打卡未达到半天次数，故计算天数0天";
                }
            }
        }
        careNurseRecords.setCalDays(day);
        careNurseRecords.setCalDayReason(reason);
        careNurseRecords.setClockNum(count);
        return careNurseRecords;
    }

    private boolean isMorning(Date date) {
        // 根据日期对象判断是否为上午
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        int hour = cal.get(Calendar.HOUR_OF_DAY);
        return hour < 12;
    }
    /**
     * 修改护理任务打卡
     * 
     * @param careNurseClocks 护理任务打卡
     * @return 结果
     */
    @Override
    public int updateCareNurseClocks(CareNurseClocks careNurseClocks)
    {
        //判断当前日期是否可以打卡，非残疾人出院前两天不再护理，残疾人截至到出院当天,若无法打卡，则抛出异常
        CareApplications careApplications = careApplicationsMapper.selectCareApplicationsByApplyId(Long.parseLong(careNurseClocks.getApplyId()));
        String leaveTimeStr = careApplications.getLeaveTime();

        // 定义日期格式化器
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

        // 将字符串转换为LocalDateTime对象
        LocalDateTime leaveTime = LocalDateTime.parse(leaveTimeStr, formatter);
        LocalDateTime currentTime = LocalDateTime.parse(careNurseClocks.getNursingTime(), formatter);

        boolean isDisabled = !careApplications.getDisability().equals("N");

        if (isDisabled) {
            // 残疾人：截至到出院当天
            if (currentTime.isAfter(leaveTime)) {
                throw new ServiceException("无法打卡：当前时间已超过出院时间!");
            }
        } else {
            // 非残疾人：出院前两天不再护理
            LocalDateTime cutoffTime = leaveTime.minus(2, ChronoUnit.DAYS);
            if (currentTime.isAfter(cutoffTime)) {
                throw new ServiceException("无法打卡：非残疾人出院前两天不再提供护理服务!");
            }
        }

        //若特困人员在中转或者擅自离院，不提供护理
        CarePersons carePersons=carePersonsMapper.selectCarePersonsByPersonId(careApplications.getPersonId());
        if(carePersons.getStatus().equals("02")){
            throw new ServiceException("无法打卡：当前特困人员处于中转状态!");
        }else if(carePersons.getStatus().equals("03")){
            throw new ServiceException("无法打卡：当前特困人员处于擅自离院状态!");
        }
        String psths[]=careNurseClocks.getImgPaths().split(",");
        int result=0;

        careNurseClocks.setUpdateTime(DateUtils.getNowDate());
        careNurseClocks.setImgPaths(careNurseClocks.getImgPaths().replaceAll("/temp/","/upload/"));
        // 判断当天任务是否一开始
        String  date=careNurseClocks.getNursingTime().substring(0,10);
        CareNurseRecords careNurseRecords= new CareNurseRecords();
        careNurseRecords.setNursingTime(date);
        careNurseRecords.setApplyId(Long.parseLong(careNurseClocks.getApplyId()));
        List<CareNurseRecords> lst=careNurseRecordsMapper.selectCareNurseRecordsList(careNurseRecords);
        if(lst.size()>0){
            CareNurseRecords careNurseRecord=lst.get(0);
            //计算计时天数
            try {
                careNurseRecord=calDayApplyRecord(careNurseRecord,careNurseClocks.getNursingTime());
            } catch (ParseException e) {
                throw new RuntimeException(e);
            }
            careNurseRecordsMapper.updateCareNurseRecords(careNurseRecord);

            //已经开始了直接新增打卡记录
            result= careNurseClocksMapper.updateCareNurseClocks(careNurseClocks);
        }else{
            //修改申请状态为正在进行，并添加护理记录及打卡记录
            careApplications.setDealStatus("in_progress");
            careApplicationsMapper.updateCareApplications(careApplications);

            careNurseRecords=new CareNurseRecords();
            careNurseRecords.setNursingTime(careNurseClocks.getNursingTime().substring(0,10));
            careNurseRecords.setApplyId(Long.parseLong(careNurseClocks.getApplyId()));
            careNurseRecordsMapper.insertCareNurseRecords(careNurseRecords);

            //获取打卡次数
            CareNurseRecords recordCount=new CareNurseRecords();
            recordCount.setApplyId(Long.parseLong(careNurseClocks.getApplyId()));
            CareRecords curr=new CareRecords();
            curr.setApplyId(Long.parseLong(careNurseClocks.getApplyId()));
            curr.setState("0");
            List<CareRecords> list=careRecordsMapper.selectCareRecordsList(curr);
            if(list.size()>0){
                curr=list.get(0);
                curr.setRemark("已打卡"+careNurseRecordsMapper.selectCareNurseRecordsList(recordCount).size()+"次");
                careRecordsMapper.updateCareRecords(curr);
            }

            result=careNurseClocksMapper.updateCareNurseClocks(careNurseClocks);
        }

        //保存成功后 将文件转移到正式路径

        for(int i=0;i<psths.length;i++){
            if(psths[i].indexOf("/temp")>-1){
                String path= RuoYiConfig.getProfile()+psths[i].replace("profile","");
                String finalFilePath=RuoYiConfig.getUploadPath()+psths[i].replace("/profile/temp","");
                try {
                    moveFileToFinalPath(path,finalFilePath);
                } catch (IOException e) {
                    throw new RuntimeException(e);
                }
            }

        }
        return result;
    }

    /**
     * 批量删除护理任务打卡
     * 
     * @param clockIds 需要删除的护理任务打卡主键
     * @return 结果
     */
    @Override
    public int deleteCareNurseClocksByClockIds(Long[] clockIds)
    {
        return careNurseClocksMapper.deleteCareNurseClocksByClockIds(clockIds);
    }

    /**
     * 删除护理任务打卡信息
     * 
     * @param clockId 护理任务打卡主键
     * @return 结果
     */
    @Override
    public int deleteCareNurseClocksByClockId(Long clockId)
    {
        return careNurseClocksMapper.deleteCareNurseClocksByClockId(clockId);
    }

    private void moveFileToFinalPath(String tempFilePath, String finalFilePath) throws IOException {
        // 规范化路径
        Path tempPath = Paths.get(tempFilePath).normalize();
        Path finalPath = Paths.get(finalFilePath).normalize();

        // 检查临时文件是否存在
        if (!Files.exists(tempPath)) {
            throw new IOException("临时文件不存在, 请重新上传: " + tempPath.toString());
        }

        // 创建目标目录（如果不存在）
        Path finalDir = finalPath.getParent();
        if (finalDir != null && !Files.exists(finalDir)) {
            Files.createDirectories(finalDir);
        }

        // 移动文件到最终路径
        try {
            Files.move(tempPath, finalPath, StandardCopyOption.REPLACE_EXISTING);
            System.out.println("文件已成功移动到: " + finalPath.toString());
        } catch (IOException e) {
            throw new IOException("文件移动失败: " + e.getMessage());
        }
    }
}
